@@ -23,6 +23,7 @@ module Senna
   INDEX_WITH_VACUUM             =      0x8000
 
   SYM_MAX_KEY_SIZE              =      0x2000
+  SYM_WITH_SIS                  =  0x80000000
 
   SNIP_NORMALIZE                =      0x0001
   SNIP_MAP_HTMLENCODE           =          -1
@@ -271,6 +272,7 @@ module Senna
   end
 
   class Sym
+    attr_reader :handle, :path
     def Sym.callback(handle)
       proc { Senna::sen_sym_close(handle) }
     end
@@ -316,8 +318,8 @@ module Senna
     end
 
     def info
-      inf = Senna::sen_sym_info(handle)
-      return {'key_size' => inf[0], 'flags' => inf[1], 'encoding' => inf[2], 'nrecords' => inf[3]}
+      inf = Senna::sen_sym_info(@handle)
+      return {'key_size' => inf[1], 'flags' => inf[2], 'encoding' => inf[3], 'nrecords' => inf[4], 'file_size' => inf[5]}
     end
 
     def get(key)
@@ -381,7 +383,17 @@ module Senna
     end
   end
 
+  class Element
+    attr_reader :phandle, :handle, :key, :value
+    def initialize(phandle, handle)
+      @phandle = phandle
+      @handle = handle
+      @key, @value = Senna::sen_record_info(@phandle, @handle)
+    end
+  end
+
   class Set
+    attr_reader :handle
     def Set.callback(handle)
       proc { Senna::sen_set_close(handle) }
     end
@@ -410,11 +422,20 @@ module Senna
       return { 'key_size' => inf[0], 'value_size' => inf[1], 'n_entries' => inf[2] }
     end
 
-    def get(key)
-      eh, _ = Senna::sen_set_get(@handle, key)
+    def []=(key, value)
+      return get(key, value)
+    end
+
+    def get(key, value)
+      eh, _ = Senna::sen_set_get(@handle, key, value)
       return eh
     end
 
+    def [](key)
+      _, value = Senna::sen_set_at(@handle, key)
+      return value
+    end
+
     def at(key)
       eh, _ = Senna::sen_set_at(@handle, key)
       return eh
@@ -677,7 +698,8 @@ module Senna
     end
 
     def rest
-      Senna::sen_query_rest(@handle)
+      len, _ = Senna::sen_query_rest(@handle, 0)
+      return Senna::sen_query_rest(@handle, len)[1]
     end
 
     def exec(index, records=nil, op=SEL_OR)
