@@ -854,15 +854,15 @@ sen_index_remove(const char *path)
   char buffer[PATH_MAX];
   if (!path || strlen(path) > PATH_MAX - 8) { return sen_invalid_argument; }
   snprintf(buffer, PATH_MAX, "%s.SEN", path);
-  sen_io_remove(buffer);
+  sen_sym_remove(buffer);
   snprintf(buffer, PATH_MAX, "%s.SEN.i", path);
-  sen_io_remove(buffer);
+  sen_io_remove(buffer); // sen_inv_remove
   snprintf(buffer, PATH_MAX, "%s.SEN.i.c", path);
   sen_io_remove(buffer);
   snprintf(buffer, PATH_MAX, "%s.SEN.l", path);
-  sen_io_remove(buffer);
+  sen_sym_remove(buffer);
   snprintf(buffer, PATH_MAX, "%s.SEN.v", path);
-  sen_io_remove(buffer);
+  sen_io_remove(buffer); // sen_store_remove
   return sen_success;
 }
 
@@ -996,7 +996,6 @@ index_add(sen_index *i, const void *key, const char *value, size_t value_len)
   sen_set_cursor *c;
   sen_inv_updspec **u;
   sen_id rid, tid, *tp;
-  int len = strlen(value);
   sen_rc r, rc = sen_success;
   sen_store_buf *sbuf = NULL;
   //  sen_log("add > (%x:%x)", i, key);
@@ -1004,7 +1003,7 @@ index_add(sen_index *i, const void *key, const char *value, size_t value_len)
   if (!(lex = sen_lex_open(i->lexicon, value, value_len, SEN_LEX_ADD|SEN_LEX_UPD))) {
     return sen_memory_exhausted;
   }
-  if (i->store) { sbuf = sen_store_buf_open(len); }
+  if (i->store) { sbuf = sen_store_buf_open(value_len); }
   h = sen_set_open(sizeof(sen_id), sizeof(sen_inv_updspec *), 0);
   if (!h) {
     SEN_LOG(sen_log_alert, "sen_set_open on index_add failed !");
@@ -1048,7 +1047,7 @@ index_add(sen_index *i, const void *key, const char *value, size_t value_len)
   }
   sen_set_cursor_close(c);
   sen_set_close(h);
-  //  sen_log("add < (%x:%d:%d) %d", key, rid, *((int *)key), strlen(value));
+  //  sen_log("add < (%x:%d:%d) %d", key, rid, *((int *)key), value_len);
   return rc;
 exit:
   sen_set_close(h);
@@ -2688,3 +2687,33 @@ sen_records_heap_close(sen_records_heap *h)
   SEN_FREE(h);
   return sen_success;
 }
+
+sen_rc
+sen_info(char **version,
+         char **configure_options,
+         char **config_path,
+         sen_encoding *default_encoding,
+         unsigned int *initial_n_segments,
+         unsigned int *partial_match_threshold)
+{
+  if (version) {
+    *version = PACKAGE_VERSION;
+  }
+  if (configure_options) {
+    *configure_options = CONFIGURE_OPTIONS;
+  }
+  if (config_path) {
+    *config_path = CONF_PATH;
+  }
+  if (default_encoding) {
+    *default_encoding = sen_default_encoding;
+  }
+  if (initial_n_segments) {
+    *initial_n_segments = sen_initial_n_segments;
+  }
+  if (partial_match_threshold) {
+    *partial_match_threshold = sen_query_escalation_threshold;
+  }
+  return sen_success;
+}
+
