@@ -32,6 +32,8 @@
 typedef struct _sen_set_element entry;
 typedef struct _sen_set_element_str entry_str;
 
+#define SEN_SET_STRHASH(e) (((entry_str *)(e))->key)
+
 inline static entry *
 entry_new(sen_set *set)
 {
@@ -112,7 +114,7 @@ sen_set_reset(sen_set * set, uint32_t ne)
     for (j = set->max_offset + 1, sp = set->index; j; j--, sp++) { 
       e = *sp; 
       if (!e || (e == GARBAGE)) { continue; } 
-      for (i = ((entry_str *)e)->key; ; i += 1048573) { 
+      for (i = SEN_SET_STRHASH(e); ; i += 1048573) { 
         dp = index + (i & m); 
         if (!*dp) { break; } 
       } 
@@ -199,7 +201,7 @@ sen_set_str_at(sen_set *set, const char *key, void **value)
   uint32_t h = str_hash((unsigned char *)key), i, m = set->max_offset;
   for (i = h; ep = index + (i & m), (e = *ep); i += 1048573) {
     if (e == GARBAGE) { continue; }
-    if (((entry_str *)e)->key == h && !strcmp(key, SEN_SET_STRKEY(e))) {
+    if (SEN_SET_STRHASH(e) == h && !strcmp(key, SEN_SET_STRKEY(e))) {
       if (value) { *value = SEN_SET_STRVAL(e); }
       return (sen_set_eh *) ep;
     }
@@ -274,7 +276,7 @@ sen_set_str_get(sen_set *set, const char *key, void **value)
     if (e == GARBAGE) {
       if (!np) { np = ep; }
     } else {
-      if (((entry_str *)e)->key == h && !strcmp(key, SEN_SET_STRKEY(e))) { goto exit; }
+      if (SEN_SET_STRHASH(e) == h && !strcmp(key, SEN_SET_STRKEY(e))) { goto exit; }
     }
   }
   if (np) {
@@ -282,13 +284,12 @@ sen_set_str_get(sen_set *set, const char *key, void **value)
     ep = np;
   }
   e = entry_new(set);
-  ((entry_str *)e)->key = h;
-  e->key = h;
+  SEN_SET_STRHASH(e) = h;
   SEN_SET_STRKEY(e) = SEN_STRDUP(key);
   *ep = e;
   set->n_entries++;
 exit :
-  if (value) { *value = e->dummy; }
+  if (value) { *value = SEN_SET_STRVAL(e); }
   return ep;
 }
 
@@ -786,7 +787,9 @@ sen_set_difference(sen_set *a, sen_set *b)
 sen_rc
 sen_set_array_init(sen_set *set, uint32_t size)
 {
+  SEN_ASSERT(!set->n_entries);
   set->arrayp = 1;
+  set->curr_entry = 0;
   if (set->chunks[SEN_SET_MAX_CHUNK]) {
     SEN_FREE(set->chunks[SEN_SET_MAX_CHUNK]);
   }
