@@ -583,7 +583,7 @@ sen_rc
 sen_sym_remove(const char *path)
 {
   if (!path) { return sen_invalid_argument; }
-  unlink(path);
+  sen_io_remove(path);
   return sen_success;
 }
 
@@ -1003,8 +1003,9 @@ sen_sym_size(sen_sym *sym)
 const char *
 _sen_sym_key(sen_sym *sym, sen_id id)
 {
+  pat_node *node;
   if (sym->v08p) { return _sen_sym_key08(sym, id); }
-  pat_node *node = pat_at(sym, id);
+  node = pat_at(sym, id);
   if (!node) { return NULL; }
   return (const char *) pat_node_get_key(sym, node);
 }
@@ -1026,8 +1027,9 @@ sen_sym_key(sen_sym *sym, sen_id id, void *keybuf, int bufsize)
 int
 sen_sym_pocket_get(sen_sym *sym, sen_id id)
 {
+  pat_node *node;
   if (sym->v08p) { return sen_sym_pocket_get08(sym, id); }
-  pat_node *node = pat_at(sym, id);
+  node = pat_at(sym, id);
   if (!node) { return -1; }
   return PAT_PKT(node);
 }
@@ -1035,8 +1037,9 @@ sen_sym_pocket_get(sen_sym *sym, sen_id id)
 sen_rc
 sen_sym_pocket_set(sen_sym *sym, sen_id id, unsigned int value)
 {
+  pat_node *node;
   if (sym->v08p) { return sen_sym_pocket_set08(sym, id, value); }
-  pat_node *node = pat_at(sym, id);
+  node = pat_at(sym, id);
   if (value > PAT_POCKET_MAX || !node) { return sen_invalid_argument; }
   SET_PAT_PKT(node, value);
   return sen_success;
