@@ -195,6 +195,7 @@ struct _sen_records {
 
 struct _sen_value {
   const char *str;
+  unsigned int str_len;
   unsigned int weight;
 };
 
@@ -249,10 +250,18 @@ struct _sen_logger_info {
   void *func_arg;
 };
 
+typedef int (*query_term_callback)(const char *, unsigned int, void *);
+
 /******** basic API ********/
 
 sen_rc sen_init(void);
 sen_rc sen_fin(void);
+sen_rc sen_info(char **version,
+                char **configure_options,
+                char **config_path,
+                sen_encoding *default_encoding,
+                unsigned int *initial_n_segments,
+                unsigned int *partial_match_threshold);
 
 sen_index *sen_index_create(const char *path, int key_size, int flags,
                             int initial_n_segments, sen_encoding encoding);
@@ -324,9 +333,10 @@ sen_set * sen_index_related_terms(sen_index *index, const char *string,
 sen_query *sen_query_open(const char *str, unsigned int str_len,
                           sen_sel_operator default_op,
                           int max_exprs, sen_encoding encoding);
-const char *sen_query_rest(sen_query *q);
+unsigned int sen_query_rest(sen_query *q, const char ** const rest);
 sen_rc sen_query_close(sen_query *q);
 sen_rc sen_query_exec(sen_index *i, sen_query *q, sen_records *r, sen_sel_operator op);
+void sen_query_term(sen_query *q, query_term_callback func, void *func_arg);
 
 sen_rc sen_index_del(sen_index *i, const void *key);
 
